@@ -19,7 +19,7 @@
 import math
 import operator
 from functools import reduce
-from typing import Any, Callable
+from typing import Any, Callable, Union
 
 import numpy as np
 import PIL
@@ -331,10 +331,10 @@ def lab_to_rgb(
     ----------
     img : PIL.Image.Image
         Input image in Lab space.
-    illuminant : {“A”, “D50”, “D55”, “D65”, “D75”, “E”}, optional
+    illuminant : {"A", "B", "C", "D50", "D55", "D65", "D75", "E"}, optional
         The name of the illuminant (the function is NOT case sensitive). Default is
         "D65".
-    observer : {“2”, “10”}, optional
+    observer : {"2", "10", "R"}, optional
         The aperture angle of the observer. Default is "2".
 
     Returns
@@ -370,7 +370,7 @@ def local_equalization(img: PIL.Image.Image, disk_size: int = 50) -> PIL.Image.I
     if len(np.array(img).shape) != 2:
         raise ValueError("Input must be 2D.")
     local_equ = sk_filters.rank.equalize(
-        np.array(img), selem=sk_morphology.disk(disk_size)
+        np.array(img), footprint=sk_morphology.disk(disk_size)
     )
     return np_to_pil(local_equ)
 
@@ -413,7 +413,9 @@ def rag_threshold(
     n_segments: int = 800,
     compactness: float = 10.0,
     threshold: int = 9,
-) -> PIL.Image.Image:
+    mask: np.ndarray = None,
+    return_labels: bool = False,
+) -> Union[PIL.Image.Image, np.ndarray]:
     """Combine similar K-means segmented regions based on threshold value.
 
     Segment an image with K-means, build region adjacency graph based on
@@ -430,12 +432,23 @@ def rag_threshold(
         Color proximity versus space proximity factor. Default is 10.0.
     threshold : int, optional
         Threshold value for combining regions. Default is 9.
+    mask : np.ndarray, optional
+        If provided, superpixels are computed only where mask is True,
+        and seed points are homogeneously distributed over the mask
+        using a K-means clustering strategy (See skimage).
+        Must be the same size as ``img``.
+    return_labels : bool, optional
+        If True, returns a labeled array where the value denotes segment
+        membership. Otherwise, returns a PIL image where each segment is colored
+        by the average color in it. Default is False.
 
     Returns
     -------
-    PIL.Image.Image
+    PIL.Image.Image, if not ``return_labels``
         Each segment has been colored based on the average
         color for that segment (and similar segments have been combined).
+    np.ndarray, if ``return_labels``
+        Value denotes segment membership.
 
     Raises
     ------
@@ -445,9 +458,17 @@ def rag_threshold(
     if img.mode == "RGBA":
         raise ValueError("Input image cannot be RGBA")
     img_arr = np.array(img)
-    labels = sk_segmentation.slic(img_arr, n_segments, compactness, start_label=0)
+    labels = sk_segmentation.slic(
+        img_arr,
+        n_segments,
+        compactness,
+        mask=mask,
+        start_label=0 if mask is None else 1,
+    )
     green = sk_future.graph.rag_mean_color(img_arr, labels)
     labels2 = sk_future.graph.cut_threshold(labels, green, threshold)
+    if return_labels:
+        return labels2
     rag = sk_color.label2rgb(labels2, img_arr, kind="avg", bg_label=-1)
     return np_to_pil(rag)
 
@@ -511,7 +532,6 @@ def rgb_to_hed(img: PIL.Image.Image) -> np.ndarray:
 
     img_arr = np.array(img)
     hed_arr = sk_color.rgb2hed(img_arr)
-    hed_arr[hed_arr < 0] = 0  # remove this after skimage fixes `rgb2hed()`
     return hed_arr
 
 
@@ -552,10 +572,10 @@ def rgb_to_lab(
     ----------
     img : PIL.Image.Image
         Input image
-    illuminant : {“A”, “D50”, “D55”, “D65”, “D75”, “E”}, optional
+    illuminant : {"A", "B", "C", "D50", "D55", "D65", "D75", "E"}, optional
         The name of the illuminant (the function is NOT case sensitive). Default is
         "D65".
-    observer : {“2”, “10”}, optional
+    observer : {"2", "10", "R"}, optional
         The aperture angle of the observer. Default is "2".
 
     Returns
