@@ -26,7 +26,13 @@ import openslide
 import PIL
 from skimage.measure import find_contours
 
-from .exceptions import LevelError, SlidePropertyError
+from .exceptions import (
+    HistolabException,
+    LevelError,
+    MayNeedLargeImageError,
+    SlidePropertyError,
+    TileSizeOrCoordinatesError,
+)
 from .filters.compositions import FiltersComposition
 from .tile import Tile
 from .types import CoordinatePair
@@ -35,8 +41,19 @@ from .util import lazyproperty
 if TYPE_CHECKING:
     from .masks import BinaryMask
 
+try:
+    from io import BytesIO
+
+    import large_image
+
+    LARGEIMAGE_IS_INSTALLED = True
+except (ModuleNotFoundError, ImportError):  # pragma: no cover
+    LARGEIMAGE_IS_INSTALLED = False
 
 IMG_EXT = "png"
+IMG_UPSAMPLE_MODE = PIL.Image.BICUBIC
+IMG_DOWNSAMPLE_MODE = PIL.Image.LANCZOS
+TILE_SIZE_PIXEL_TOLERANCE = 5
 
 
 class Slide:
@@ -48,21 +65,48 @@ class Slide:
         Path where the WSI is saved.
     processed_path : Union[str, pathlib.Path]
         Path where the tiles will be saved to.
+    use_largeimage : bool, optional
+        Whether or not to use the `large_image` package for accessing the
+        slide and extracting or calculating various metadata. If this is
+        `False`, `openslide` is used. If it is `True`, `large_image` will try
+        from the various installed tile sources. For example, if you installed
+        it using `large_image[all]`, it will try `openslide` first, then `PIL`,
+        and so on, depending on the slide format and metadata. `large_image`
+        also handles internal logic to enable fetching exact micron-per-pixel
+        resolution tiles by interpolating between the internal levels of the
+        slide. If you don't mind installing an extra dependency,
+        we recommend setting this to True and fetching Tiles at exact
+        resolutions as opposed to levels. Different scanners have different
+        specifications, and the same level may not always encode the same
+        magnification in different scanners and slide formats.
 
     Raises
     ------
     TypeError
         If the processed path is not specified.
+    ModuleNotFoundError
+        when `use_largeimage` is set to True and `large_image` module is not
+        installed.
     """
 
     def __init__(
-        self, path: Union[str, pathlib.Path], processed_path: Union[str, pathlib.Path]
+        self,
+        path: Union[str, pathlib.Path],
+        processed_path: Union[str, pathlib.Path],
+        use_largeimage: bool = False,
     ) -> None:
         self._path = str(path) if isinstance(path, pathlib.Path) else path
 
         if processed_path is None:
             raise TypeError("processed_path cannot be None.")
         self._processed_path = processed_path
+        if use_largeimage and not LARGEIMAGE_IS_INSTALLED:  # pragma: no cover
+            raise ModuleNotFoundError(
+                "Setting use_large_image to True requires installation "
+                "of the large_image module. Please visit: "
+                "https://github.com/girder/large_image for instructions."
+            )
+        self._use_largeimage = use_largeimage
 
     def __repr__(self):
         return (
@@ -72,6 +116,51 @@ class Slide:
 
     # ---public interface methods and properties---
 
+    @lazyproperty
+    def base_mpp(self) -> float:
+        """Get microns-per-pixel resolution at scan magnification.
+
+        Returns
+        -------
+        float
+            Microns-per-pixel resolution at scan (base) magnification.
+
+        Raises
+        ------
+        ValueError
+            If `large_image` cannot detemine the slide magnification.
+        MayNeedLargeImageError
+            If `use_largeimage` was set to False when slide was initialized,
+            and we cannot determine the magnification otherwise.
+        """
+        if self._use_largeimage:
+            if self._metadata.get("mm_x") is not None:
+                return self._metadata["mm_x"] * (10 ** 3)
+            raise ValueError(
+                "Unknown scan resolution! This slide is missing metadata "
+                "needed for calculating the scanning resolution. Without "
+                "this information, you can only ask for a tile by level, "
+                "not mpp resolution."
+            )
+
+        if "openslide.mpp-x" in self.properties:
+            return float(self.properties["openslide.mpp-x"])
+
+        if "aperio.MPP" in self.properties:
+            return float(self.properties["aperio.MPP"])
+
+        if (
+            "tiff.XResolution" in self.properties
+            and self.properties.get("tiff.ResolutionUnit") == "centimeter"
+        ):
+            return 1e4 / float(self.properties["tiff.XResolution"])
+
+        raise MayNeedLargeImageError(
+            "Unknown scan magnification! This slide format may be best "
+            "handled using the large_image module. Consider setting "
+            "use_largeimage to True when instantiating this Slide."
+        )
+
     @lazyproperty
     def dimensions(self) -> Tuple[int, int]:
         """Slide dimensions (w,h) at level 0.
@@ -81,10 +170,17 @@ class Slide:
         dimensions : Tuple[int, int]
             Slide dimensions (width, height)
         """
+        if self._use_largeimage:
+            return self._metadata["sizeX"], self._metadata["sizeY"]
+
         return self._wsi.dimensions
 
     def extract_tile(
-        self, coords: CoordinatePair, level: int, tile_size: Tuple
+        self,
+        coords: CoordinatePair,
+        tile_size: Tuple[int, int],
+        level: int = None,
+        mpp: float = None,
     ) -> Tile:
         """Extract a tile of the image at the selected level.
 
@@ -92,31 +188,84 @@ class Slide:
         ----------
         coords : CoordinatePair
             Coordinates at level 0 from which to extract the tile.
+        tile_size : Tuple[int, int]
+            Final size of the extracted tile (x,y). If you choose to specify
+            the `mpp` argument, you may elect to set this as `None` to return
+            the tile as-is from `large_image` without any resizing. This is not
+            recommended, as tile size may be off by a couple of pixels when
+            coordinates are mapped to the exact mpp you request.
         level : int
-            Level from which to extract the tile.
-        tile_size: tuple
-            Final size of the tile (x,y).
+            Level from which to extract the tile. If you specify this, and
+            `mpp` is None, `openslide` will be used to fetch tiles from this
+            level from the slide. `openslide` is used for fetching tiles by
+            level, regardless of `self.use_largeimage`.
+        mpp : float
+            Micron per pixel resolution. Takes precedence over level. If this
+            is not None, `large_image` will be used to fetch tiles at the exact
+            microns-per-pixel resolution requested.
 
         Returns
         -------
         tile : Tile
             Image containing the selected tile.
         """
+        if level is None and mpp is None:
+            raise ValueError("Either level or mpp must be provided!")
+
+        if level is not None:
+            level = level if level >= 0 else self._remap_level(level)
 
-        level = level if level >= 0 else self._remap_level(level)
         if not self._has_valid_coords(coords):
             # OpenSlide doesn't complain if the coordinates for extraction are wrong,
             # but it returns an odd image.
-            raise ValueError(
+            raise TileSizeOrCoordinatesError(
                 f"Extraction Coordinates {coords} not valid for slide with dimensions "
                 f"{self.dimensions}"
             )
 
-        image = self._wsi.read_region(
-            location=(coords.x_ul, coords.y_ul), level=level, size=tile_size
-        )
-        tile = Tile(image, coords, level)
-        return tile
+        if mpp is None:
+            image = self._wsi.read_region(
+                location=(coords.x_ul, coords.y_ul), level=level, size=tile_size
+            )
+        else:
+            mm = mpp / 1000
+            image, _ = self._tile_source.getRegion(
+                region=dict(
+                    left=coords.x_ul,
+                    top=coords.y_ul,
+                    right=coords.x_br,
+                    bottom=coords.y_br,
+                    units="base_pixels",
+                ),
+                scale=dict(mm_x=mm, mm_y=mm),
+                format=large_image.tilesource.TILE_FORMAT_PIL,
+                jpegQuality=100,
+            )
+            # Sometimes when mpp kwarg is used, the image size is off from
+            # what the user expects by a couple of pixels
+            if tile_size is not None and not tile_size == image.size:
+                if any(
+                    np.abs(tile_size[i] - j) > TILE_SIZE_PIXEL_TOLERANCE
+                    for i, j in enumerate(image.size)
+                ):
+                    raise RuntimeError(
+                        f"The tile you requested at a resolution of {mpp} MPP "
+                        f"has a size of {image.size}, yet you specified a "
+                        f"final `tile_size` of {tile_size}, which is a very "
+                        "different value. When you set `mpp`, the `tile_size` "
+                        "parameter is used to resize fetched tiles if they "
+                        f"are off by just {TILE_SIZE_PIXEL_TOLERANCE} pixels "
+                        "due to rounding differences etc. Please check if you "
+                        "requested the right `mpp` and/or `tile_size`."
+                    )
+                image = image.resize(
+                    tile_size,
+                    IMG_UPSAMPLE_MODE
+                    if tile_size[0] >= image.size[0]
+                    else IMG_DOWNSAMPLE_MODE,
+                )
+
+        return Tile(image, coords, level)
 
     def level_dimensions(self, level: int = 0) -> Tuple[int, int]:
         """Return the slide dimensions (w,h) at the specified level
@@ -254,7 +403,7 @@ class Slide:
         # pad the mask to have closed contours along the edges
         padded_mask = np.pad(resized_mask, pad_width=1, mode="constant")
         contours = [
-            cont - 1 for cont in find_contours(padded_mask, 0.5)
+            cont - 1 for cont in find_contours(padded_mask, level=0.5)
         ]  # unpad countours
 
         for contour in contours:
@@ -347,10 +496,33 @@ class Slide:
         PIL.Image.Image
             The slide thumbnail.
         """
+        if self._use_largeimage:
+            thumb_bytes, _ = self._tile_source.getThumbnail(encoding="PNG")
+            thumbnail = self._bytes2pil(thumb_bytes).convert("RGB")
+            return thumbnail
+
         return self._wsi.get_thumbnail(self._thumbnail_size)
 
     # ------- implementation helpers -------
 
+    @staticmethod
+    def _bytes2pil(bytesim: bytearray):
+        """Convert a bytes image to a PIL image object.
+
+        Parameters
+        ----------
+        bytesim : bytearray
+            A bytes object representation of an image.
+
+        Returns
+        -------
+        PIL.Image.Image
+            A PIL Image object converted from the Bytes input.
+        """
+        image_content = BytesIO(bytesim)
+        image_content.seek(0)
+        return PIL.Image.open(image_content)
+
     def _has_valid_coords(self, coords: CoordinatePair) -> bool:
         """Check if ``coords`` are valid 0-level coordinates.
 
@@ -371,6 +543,19 @@ class Slide:
             and 0 <= coords.y_br < self.dimensions[1]
         )
 
+    @lazyproperty
+    def _metadata(self) -> dict:
+        """Get metadata about this slide, including magnification.
+
+        Returns
+        -------
+        dict
+           This function is a wrapper. Please read the documentation for
+           ``large_image.TileSource.getMetadata()`` for details on the return
+           keys and data types.
+        """
+        return self._tile_source.getMetadata()
+
     def _remap_level(self, level: int) -> int:
         """Remap negative index for the given level onto a positive one.
 
@@ -418,13 +603,31 @@ class Slide:
         """
 
         _, _, new_w, new_h = self._resampled_dimensions(scale_factor)
-        level = self._wsi.get_best_level_for_downsample(scale_factor)
-        whole_slide_image = self._wsi.read_region(
-            (0, 0), level, self._wsi.level_dimensions[level]
-        )
+        if self._use_largeimage:
+            kwargs = (
+                {
+                    "scale": {
+                        "magnification": self._metadata["magnification"] / scale_factor
+                    }
+                }
+                if self._metadata["magnification"] is not None
+                else {}
+            )
+            wsi_image, _ = self._tile_source.getRegion(
+                format=large_image.tilesource.TILE_FORMAT_PIL,
+                **kwargs,
+            )
+        else:
+            level = self._wsi.get_best_level_for_downsample(scale_factor)
+            wsi_image = self._wsi.read_region(
+                (0, 0), level, self._wsi.level_dimensions[level]
+            )
         # ---converts openslide read_region to an actual RGBA image---
-        whole_slide_image = whole_slide_image.convert("RGB")
-        img = whole_slide_image.resize((new_w, new_h), PIL.Image.BILINEAR)
+        wsi_image = wsi_image.convert("RGB")
+        img = wsi_image.resize(
+            (new_w, new_h),
+            IMG_UPSAMPLE_MODE if new_w >= wsi_image.size[0] else IMG_DOWNSAMPLE_MODE,
+        )
         arr_img = np.asarray(img)
         return img, arr_img
 
@@ -477,6 +680,7 @@ class Slide:
         Tuple[int, int]
             Thumbnail size
         """
+
         return tuple(
             [
                 int(s / np.power(10, math.ceil(math.log10(s)) - 3))
@@ -484,6 +688,29 @@ class Slide:
             ]
         )
 
+    @lazyproperty
+    def _tile_source(self) -> Union[openslide.OpenSlide, openslide.ImageSlide]:
+        """Open the slide and returns a large_image tile source object
+
+        Returns
+        -------
+        source : large_image TileSource object
+            An TileSource object representing a whole-slide image.
+
+        Raises
+        ------
+        MayNeedLargeImageError
+            If `use_largeimage` was set to False when slide was initialized.
+        """
+        if not self._use_largeimage:
+            raise MayNeedLargeImageError(
+                "This property uses the large_image module. Please set "
+                "use_largeimage to True when instantiating this Slide."
+            )
+
+        source = large_image.getTileSource(self._path)
+        return source
+
     @lazyproperty
     def _wsi(self) -> Union[openslide.OpenSlide, openslide.ImageSlide]:
         """Open the slide and returns an openslide object
@@ -493,16 +720,21 @@ class Slide:
         slide : OpenSlide object
             An OpenSlide object representing a whole-slide image.
         """
+        bad_format_error = (
+            "This slide may be corrupted or have a non-standard format not "
+            "handled by the openslide and PIL libraries. Consider setting "
+            "use_largeimage to True when instantiating this Slide."
+        )
         try:
             slide = openslide.open_slide(self._path)
         except PIL.UnidentifiedImageError:
-            raise PIL.UnidentifiedImageError(
-                "Your wsi has something broken inside, a doctor is needed"
-            )
+            raise PIL.UnidentifiedImageError(bad_format_error)
         except FileNotFoundError:
             raise FileNotFoundError(
                 f"The wsi path resource doesn't exist: {self._path}"
             )
+        except Exception as other_error:
+            raise HistolabException(other_error.__repr__() + f". {bad_format_error}")
         return slide
 
 
@@ -510,11 +742,18 @@ class SlideSet:
     """Slideset object. It is considered a collection of Slides."""
 
     def __init__(
-        self, slides_path: str, processed_path: str, valid_extensions: list
+        self,
+        slides_path: str,
+        processed_path: str,
+        valid_extensions: List[str],
+        keep_slides: List[str] = None,
+        slide_kwargs: dict = None,
     ) -> None:
         self._slides_path = slides_path
         self._processed_path = processed_path
         self._valid_extensions = valid_extensions
+        self._keep_slides = keep_slides
+        self._slide_kwargs = slide_kwargs if slide_kwargs is not None else {}
 
     def __iter__(self) -> Iterator[Slide]:
         """Slides of the slideset
@@ -523,11 +762,21 @@ class SlideSet:
         -------
         generator of `Slide` objects.
         """
+        slide_names = [
+            name
+            for name in os.listdir(self._slides_path)
+            if (os.path.splitext(name)[1] in self._valid_extensions)
+        ]
+        if self._keep_slides is not None:
+            slide_names = [name for name in slide_names if name in self._keep_slides]
         return iter(
             [
-                Slide(os.path.join(self._slides_path, _path), self._processed_path)
-                for _path in os.listdir(self._slides_path)
-                if os.path.splitext(_path)[1] in self._valid_extensions
+                Slide(
+                    os.path.join(self._slides_path, name),
+                    self._processed_path,
+                    **self._slide_kwargs,
+                )
+                for name in slide_names
             ]
         )
 
