@@ -26,7 +26,7 @@ from typing import Iterable, List, Optional, Tuple, Union
 import numpy as np
 import PIL
 
-from .exceptions import LevelError, TileSizeError
+from .exceptions import LevelError, TileSizeOrCoordinatesError
 from .masks import BiggestTissueBoxMask, BinaryMask
 from .scorer import Scorer
 from .slide import Slide
@@ -57,15 +57,16 @@ class Tiler(Protocol):
     """General tiler object"""
 
     level: int
+    mpp: float  # if provided, always takes precedence over level
     tile_size: Tuple[int, int]
 
     @abstractmethod
     def extract(
         self,
         slide: Slide,
-        log_level: str,
         extraction_mask: BinaryMask = BiggestTissueBoxMask(),
-    ):
+        log_level: str = "INFO",
+    ) -> None:
         pass  # pragma: no cover
 
     def locate_tiles(
@@ -157,6 +158,23 @@ class Tiler(Protocol):
             and self.tile_size[1] <= slide.level_dimensions(self.level)[1]
         )
 
+    def _scale_factor(self, slide: Slide) -> float:
+        """Retrieve the scale factor that maps the original tile_size to proper one.
+
+        Parameters
+        ----------
+        slide : Slide
+            The slide to tile.
+
+        Returns
+        -------
+        float
+            Scale factor that maps the original self.tile_size to proper one.
+        """
+        if self.mpp is None:
+            return 1.0
+        return self.mpp / slide.base_mpp
+
     @staticmethod
     def _tile_coords_and_outline_generator(
         tiles_coords: Iterable[CoordinatePair],
@@ -228,6 +246,23 @@ class Tiler(Protocol):
     ) -> Tuple[Tile, CoordinatePair]:
         pass  # pragma: no cover
 
+    def _tile_size(self, slide: Slide) -> Tuple[int, int]:
+        """Get the proper tile size for level or mpp requested.
+
+        Parameters
+        ----------
+        slide : Slide
+            The slide to tile.
+
+        Returns
+        -------
+        Tuple[int, int]
+            Proper tile size at desired level or MPP resolution.
+        """
+        if self.mpp is None:
+            return self.tile_size
+        return tuple(int(j * self._scale_factor(slide)) for j in self.tile_size)
+
     def _validate_level(self, slide: Slide) -> None:
         """Validate the Tiler's level according to the Slide.
 
@@ -261,7 +296,7 @@ class Tiler(Protocol):
             If the tile size is larger than the slide size
         """
         if not self._has_valid_tile_size(slide):
-            raise TileSizeError(
+            raise TileSizeOrCoordinatesError(
                 f"Tile size {self.tile_size} is larger than slide size "
                 f"{slide.level_dimensions(self.level)} at level {self.level}"
             )
@@ -276,6 +311,7 @@ class GridTiler(Tiler):
         (width, height) of the extracted tiles.
     level : int, optional
         Level from which extract the tiles. Default is 0.
+        Superceded by mpp if the mpp argument is provided.
     check_tissue : bool, optional
         Whether to check if the tile has enough tissue to be saved. Default is True.
     tissue_percent : float, optional
@@ -290,6 +326,9 @@ class GridTiler(Tiler):
         Prefix to be added to the tile filename. Default is an empty string.
     suffix : str, optional
         Suffix to be added to the tile filename. Default is '.png'
+    mpp : float, optional
+        Micron per pixel resolution of extracted tiles. Takes precedence over level.
+        Default is None.
     """
 
     def __init__(
@@ -301,9 +340,12 @@ class GridTiler(Tiler):
         pixel_overlap: int = 0,
         prefix: str = "",
         suffix: str = ".png",
+        mpp: float = None,
     ):
         self.tile_size = tile_size
-        self.level = level
+        self.final_tile_size = tile_size
+        self.level = level if mpp is None else 0
+        self.mpp = mpp
         self.check_tissue = check_tissue
         self.tissue_percent = tissue_percent
         self.pixel_overlap = pixel_overlap
@@ -340,6 +382,8 @@ class GridTiler(Tiler):
         level = logging.getLevelName(log_level)
         logger.setLevel(level)
         self._validate_level(slide)
+        self.tile_size = self._tile_size(slide)
+        self.pixel_overlap = int(self._scale_factor(slide) * self.pixel_overlap)
         self._validate_tile_size(slide)
 
         grid_tiles = self._tiles_generator(slide, extraction_mask)
@@ -364,8 +408,8 @@ class GridTiler(Tiler):
 
     # ------- implementation helpers -------
 
+    @staticmethod
     def _are_coordinates_within_extraction_mask(
-        self,
         tile_thumb_coords: CoordinatePair,
         binary_mask_region: np.ndarray,
     ) -> bool:
@@ -503,38 +547,6 @@ class GridTiler(Tiler):
                 bbox_coordinates_lvl, slide, binary_mask_region
             )
 
-    def _tiles_generator(
-        self, slide: Slide, extraction_mask: BinaryMask = BiggestTissueBoxMask()
-    ) -> Tuple[Tile, CoordinatePair]:
-        """Generator of tiles arranged in a grid.
-
-        Parameters
-        ----------
-        slide : Slide
-            Slide from which to extract the tiles
-        extraction_mask : BinaryMask, optional
-            BinaryMask object defining how to compute a binary mask from a Slide.
-            Default `BiggestTissueBoxMask`.
-
-        Yields
-        -------
-        Tile
-            Extracted tile
-        CoordinatePair
-            Coordinates of the slide at level 0 from which the tile has been extracted
-        """
-        grid_coordinates_generator = self._grid_coordinates_generator(
-            slide, extraction_mask
-        )
-        for coords in grid_coordinates_generator:
-            try:
-                tile = slide.extract_tile(coords, self.level, self.tile_size)
-            except ValueError:
-                continue
-
-            if not self.check_tissue or tile.has_enough_tissue(self.tissue_percent):
-                yield tile, coords
-
     def _n_tiles_column(self, bbox_coordinates: CoordinatePair) -> int:
         """Return the number of tiles which can be extracted in a column.
 
@@ -569,6 +581,43 @@ class GridTiler(Tiler):
             self.tile_size[0] - self.pixel_overlap
         )
 
+    def _tiles_generator(
+        self, slide: Slide, extraction_mask: BinaryMask = BiggestTissueBoxMask()
+    ) -> Tuple[Tile, CoordinatePair]:
+        """Generator of tiles arranged in a grid.
+
+        Parameters
+        ----------
+        slide : Slide
+            Slide from which to extract the tiles
+        extraction_mask : BinaryMask, optional
+            BinaryMask object defining how to compute a binary mask from a Slide.
+            Default `BiggestTissueBoxMask`.
+
+        Yields
+        -------
+        Tile
+            Extracted tile
+        CoordinatePair
+            Coordinates of the slide at level 0 from which the tile has been extracted
+        """
+        grid_coordinates_generator = self._grid_coordinates_generator(
+            slide, extraction_mask
+        )
+        for coords in grid_coordinates_generator:
+            try:
+                tile = slide.extract_tile(
+                    coords,
+                    tile_size=self.final_tile_size,
+                    mpp=self.mpp,
+                    level=self.level if self.mpp is None else None,
+                )
+            except TileSizeOrCoordinatesError:
+                continue
+
+            if not self.check_tissue or tile.has_enough_tissue(self.tissue_percent):
+                yield tile, coords
+
 
 class RandomTiler(Tiler):
     """Extractor of random tiles from a Slide, at the given level, with the given size.
@@ -581,6 +630,7 @@ class RandomTiler(Tiler):
         Maximum number of tiles to extract.
     level : int, optional
         Level from which extract the tiles. Default is 0.
+        Superceded by mpp if the mpp argument is provided.
     seed : int, optional
         Seed for RandomState. Must be convertible to 32 bit unsigned integers. Default
         is 7.
@@ -597,6 +647,9 @@ class RandomTiler(Tiler):
     max_iter : int, optional
         Maximum number of iterations performed when searching for eligible (if
         ``check_tissue=True``) tiles. Must be grater than or equal to ``n_tiles``.
+    mpp : float, optional
+        Micron per pixel resolution. If provided, takes precedence over level.
+        Default is None.
     """
 
     def __init__(
@@ -610,11 +663,14 @@ class RandomTiler(Tiler):
         prefix: str = "",
         suffix: str = ".png",
         max_iter: int = int(1e4),
+        mpp: float = None,
     ):
         self.tile_size = tile_size
+        self.final_tile_size = tile_size
         self.n_tiles = n_tiles
         self.max_iter = max_iter
-        self.level = level
+        self.level = level if mpp is None else 0
+        self.mpp = mpp
         self.seed = seed
         self.check_tissue = check_tissue
         self.tissue_percent = tissue_percent
@@ -650,6 +706,7 @@ class RandomTiler(Tiler):
         level = logging.getLevelName(log_level)
         logger.setLevel(level)
         self._validate_level(slide)
+        self.tile_size = self._tile_size(slide)
         self._validate_tile_size(slide)
 
         random_tiles = self._tiles_generator(slide, extraction_mask)
@@ -759,8 +816,13 @@ class RandomTiler(Tiler):
         while True:
             tile_wsi_coords = self._random_tile_coordinates(slide, extraction_mask)
             try:
-                tile = slide.extract_tile(tile_wsi_coords, self.level, self.tile_size)
-            except ValueError:
+                tile = slide.extract_tile(
+                    tile_wsi_coords,
+                    tile_size=self.final_tile_size,
+                    mpp=self.mpp,
+                    level=self.level if self.mpp is None else None,
+                )
+            except TileSizeOrCoordinatesError:
                 iteration -= 1
                 continue
 
@@ -793,6 +855,7 @@ class ScoreTiler(GridTiler):
         will be saved (same exact behaviour of a GridTiler). Cannot be negative.
     level : int, optional
         Level from which extract the tiles. Default is 0.
+        Superceded by mpp if the mpp argument is provided.
     check_tissue : bool, optional
         Whether to check if the tile has enough tissue to be saved. Default is True.
     tissue_percent : float, optional
@@ -807,6 +870,9 @@ class ScoreTiler(GridTiler):
         Prefix to be added to the tile filename. Default is an empty string.
     suffix : str, optional
         Suffix to be added to the tile filename. Default is '.png'
+    mpp : float, optional.
+        Micron per pixel resolution. If provided, takes precedence over level.
+        Default is None.
     """
 
     def __init__(
@@ -820,6 +886,7 @@ class ScoreTiler(GridTiler):
         pixel_overlap: int = 0,
         prefix: str = "",
         suffix: str = ".png",
+        mpp: float = None,
     ):
         self.scorer = scorer
         self.n_tiles = n_tiles
@@ -832,6 +899,7 @@ class ScoreTiler(GridTiler):
             pixel_overlap,
             prefix,
             suffix,
+            mpp=mpp,
         )
 
     def extract(
@@ -869,6 +937,8 @@ class ScoreTiler(GridTiler):
         level = logging.getLevelName(log_level)
         logger.setLevel(level)
         self._validate_level(slide)
+        self.tile_size = self._tile_size(slide)
+        self.pixel_overlap = int(self._scale_factor(slide) * self.pixel_overlap)
         self._validate_tile_size(slide)
 
         highest_score_tiles, highest_scaled_score_tiles = self._tiles_generator(
@@ -879,7 +949,12 @@ class ScoreTiler(GridTiler):
         filenames = []
 
         for tiles_counter, (score, tile_wsi_coords) in enumerate(highest_score_tiles):
-            tile = slide.extract_tile(tile_wsi_coords, self.level, self.tile_size)
+            tile = slide.extract_tile(
+                tile_wsi_coords,
+                tile_size=self.final_tile_size,
+                mpp=self.mpp,
+                level=self.level if self.mpp is None else None,
+            )
             tile_filename = self._tile_filename(tile_wsi_coords, tiles_counter)
             tile.save(os.path.join(slide.processed_path, tile_filename))
             filenames.append(tile_filename)
@@ -961,8 +1036,6 @@ class ScoreTiler(GridTiler):
 
         Parameters
         ----------
-        slide : Slide
-            The slide to extract the tiles from.
         report_path : str
             Path to the report
         highest_score_tiles : List[Tuple[float, CoordinatePair]]
