@@ -7,7 +7,7 @@ import numpy as np
 import PIL
 import pytest
 
-from histolab.exceptions import LevelError, SlidePropertyError
+from histolab.exceptions import HistolabException, LevelError, SlidePropertyError
 from histolab.masks import BiggestTissueBoxMask, TissueMask
 from histolab.slide import Slide
 
@@ -26,21 +26,36 @@ class Describe_Slide:
 
         assert name == ntpath.basename(SVS.CMU_1_SMALL_REGION).split(".")[0]
 
+    @pytest.mark.parametrize("use_largeimage", (True, False))
+    def it_knows_its_base_mpp(self, use_largeimage, tmpdir):
+        slide = Slide(SVS.CMU_1_SMALL_REGION, tmpdir, use_largeimage=use_largeimage)
+
+        assert slide.base_mpp == 0.499
+
     def it_calculate_resampled_nparray_from_small_region_svs_image(self):
         slide = Slide(
             SVS.CMU_1_SMALL_REGION, os.path.join(SVS.CMU_1_SMALL_REGION, "processed")
         )
-
-        resampled_array = slide.resampled_array(scale_factor=32)
-
         expected_value = load_expectation(
             "svs-images/small-region-svs-resampled-array", type_="npy"
         )
+
+        resampled_array = slide.resampled_array(scale_factor=32)
+
         np.testing.assert_almost_equal(resampled_array, expected_value)
 
-    def it_knows_the_right_slide_dimension(self):
+    @pytest.mark.parametrize(
+        "use_largeimage",
+        [
+            (False,),
+            (True,),
+        ],
+    )
+    def it_knows_the_right_slide_dimension(self, use_largeimage):
         slide = Slide(
-            SVS.CMU_1_SMALL_REGION, os.path.join(SVS.CMU_1_SMALL_REGION, "processed")
+            SVS.CMU_1_SMALL_REGION,
+            os.path.join(SVS.CMU_1_SMALL_REGION, "processed"),
+            use_largeimage=use_largeimage,
         )
         image = PIL.Image.open(SVS.CMU_1_SMALL_REGION)
 
@@ -132,8 +147,24 @@ class Describe_Slide:
             slide._wsi
 
         assert isinstance(err.value, PIL.UnidentifiedImageError)
-        assert (
-            str(err.value) == "Your wsi has something broken inside, a doctor is needed"
+        assert str(err.value) == (
+            "This slide may be corrupted or have a non-standard format not "
+            "handled by the openslide and PIL libraries. Consider setting "
+            "use_largeimage to True when instantiating this Slide."
+        )
+
+    def it_raises_miscellaneous_error(self):
+        slide = Slide(path=None, processed_path=os.path.join(SVS.BROKEN, "processed"))
+
+        with pytest.raises(HistolabException) as err:
+            slide._wsi
+
+        assert isinstance(err.value, HistolabException)
+        assert str(err.value).replace(",", "") == (
+            "ArgumentError(\"argument 1: <class 'TypeError'>: Incorrect type\")"
+            ". This slide may be corrupted or have a non-standard format not "
+            "handled by the openslide and PIL libraries. Consider setting "
+            "use_largeimage to True when instantiating this Slide."
         )
 
     @pytest.mark.parametrize(
