@@ -8,7 +8,7 @@ from PIL import Image
 
 from histolab.masks import BiggestTissueBoxMask, TissueMask
 from histolab.scorer import NucleiScorer
-from histolab.slide import Slide
+from histolab.slide import TILE_SIZE_PIXEL_TOLERANCE, Slide
 from histolab.tiler import GridTiler, RandomTiler, ScoreTiler
 
 from ..fixtures import EXTERNAL_SVS, SVS, TIFF
@@ -128,10 +128,7 @@ class DescribeRandomTiler:
             seed=42,
             check_tissue=check_tissue,
         )
-        expected_img = load_expectation(
-            expectation,
-            type_="png",
-        )
+        expected_img = load_expectation(expectation, type_="png")
 
         tiles_location_img = random_tiles_extractor.locate_tiles(
             slide,
@@ -146,35 +143,46 @@ class DescribeRandomTiler:
         np.testing.assert_array_almost_equal(tiles_location_img, expected_img)
 
     @pytest.mark.parametrize(
-        "fixture_slide, tile_size, level, seed, n_tiles",
+        "fixture_slide, tile_size, level, seed, n_tiles, mpp, use_largeimage",
         (
             # Squared tile size
-            (SVS.TCGA_CR_7395_01A_01_TS1, (128, 128), 1, 42, 20),
-            (SVS.TCGA_CR_7395_01A_01_TS1, (128, 128), 0, 42, 10),
-            (SVS.TCGA_CR_7395_01A_01_TS1, (128, 128), 1, 2, 20),
-            (SVS.TCGA_CR_7395_01A_01_TS1, (128, 128), 0, 2, 10),
-            (TIFF.KIDNEY_48_5, (10, 10), 0, 20, 20),
-            (TIFF.KIDNEY_48_5, (20, 20), 0, 20, 10),
+            (SVS.TCGA_CR_7395_01A_01_TS1, (128, 128), 1, 42, 20, None, False),
+            (SVS.TCGA_CR_7395_01A_01_TS1, (128, 128), 0, 42, 10, None, False),
+            (SVS.TCGA_CR_7395_01A_01_TS1, (128, 128), 1, 2, 20, None, False),
+            (SVS.TCGA_CR_7395_01A_01_TS1, (128, 128), 0, 2, 10, None, False),
+            (SVS.CMU_1_SMALL_REGION, (128, 128), 0, 2, 5, 0.5, True),
+            (TIFF.KIDNEY_48_5, (10, 10), 0, 20, 20, None, False),
+            (TIFF.KIDNEY_48_5, (20, 20), 0, 20, 10, None, False),
             # Not squared tile size
-            (SVS.TCGA_CR_7395_01A_01_TS1, (135, 128), 1, 42, 20),
-            (SVS.TCGA_CR_7395_01A_01_TS1, (135, 128), 0, 42, 10),
-            (SVS.TCGA_CR_7395_01A_01_TS1, (135, 128), 1, 2, 20),
-            (TIFF.KIDNEY_48_5, (10, 20), 0, 2, 10),
-            (TIFF.KIDNEY_48_5, (20, 10), 0, 20, 20),
-            (TIFF.KIDNEY_48_5, (10, 15), 0, 20, 10),
+            (SVS.TCGA_CR_7395_01A_01_TS1, (135, 128), 1, 42, 20, None, False),
+            (SVS.TCGA_CR_7395_01A_01_TS1, (135, 128), 0, 42, 10, None, False),
+            (SVS.TCGA_CR_7395_01A_01_TS1, (135, 128), 1, 2, 20, None, False),
+            (SVS.CMU_1_SMALL_REGION, (126, 128), 0, 10, 5, 0.5, True),
+            (TIFF.KIDNEY_48_5, (10, 20), 0, 2, 10, None, False),
+            (TIFF.KIDNEY_48_5, (20, 10), 0, 20, 20, None, False),
+            (TIFF.KIDNEY_48_5, (10, 15), 0, 20, 10, None, False),
         ),
     )
     def test_extract_tiles_respecting_the_given_tile_size(
-        self, tmpdir, fixture_slide, tile_size, level, seed, n_tiles
+        self,
+        tmpdir,
+        fixture_slide,
+        tile_size,
+        level,
+        seed,
+        n_tiles,
+        mpp,
+        use_largeimage,
     ):
         processed_path = os.path.join(tmpdir, "processed")
-        slide = Slide(fixture_slide, processed_path)
+        slide = Slide(fixture_slide, processed_path, use_largeimage=use_largeimage)
         random_tiles_extractor = RandomTiler(
             tile_size=tile_size,
             n_tiles=n_tiles,
             level=level,
             seed=seed,
             check_tissue=True,
+            mpp=mpp,
         )
         binary_mask = BiggestTissueBoxMask()
 
@@ -183,10 +191,58 @@ class DescribeRandomTiler:
         for tile in os.listdir(processed_path):
             assert Image.open(os.path.join(processed_path, tile)).size == tile_size
 
+    @pytest.mark.parametrize(
+        "fixture_slide, tile_size",
+        (
+            (SVS.CMU_1_SMALL_REGION, (300, 300)),
+            (SVS.CMU_1_SMALL_REGION, (20, 15)),
+        ),
+    )
+    def test_raise_error_if_mpp_and_discordant_tile_size(
+        self, tmpdir, fixture_slide, tile_size
+    ):
+        processed_path = os.path.join(tmpdir, "processed")
+        slide = Slide(fixture_slide, processed_path, use_largeimage=True)
+        random_tiles_extractor = RandomTiler(
+            tile_size=(128, 128),
+            n_tiles=3,
+            seed=0,
+            check_tissue=True,
+            mpp=0.5,
+        )
+        random_tiles_extractor.final_tile_size = tile_size
+        binary_mask = BiggestTissueBoxMask()
+
+        with pytest.raises(RuntimeError) as err:
+            random_tiles_extractor.extract(slide, binary_mask)
+
+        assert isinstance(err.value, RuntimeError)
+        assert str(err.value) == (
+            f"The tile you requested at a resolution of 0.5 MPP "
+            f"has a size of (127, 127), yet you specified a "
+            f"final `tile_size` of {tile_size}, which is a very "
+            "different value. When you set `mpp`, the `tile_size` "
+            "parameter is used to resize fetched tiles if they "
+            f"are off by just {TILE_SIZE_PIXEL_TOLERANCE} pixels "
+            "due to rounding differences etc. Please check if you "
+            "requested the right `mpp` and/or `tile_size`."
+        )
+
+    @pytest.mark.parametrize(
+        "mpp, fixed_tile_size", ((None, (512, 512)), (0.75, (769, 769)))
+    )
+    def it_sets_tile_size_in_extract_if_mpp(self, mpp, fixed_tile_size, tmpdir):
+        slide = Slide(SVS.CMU_1_SMALL_REGION, tmpdir, use_largeimage=True)
+        tiler = RandomTiler((512, 512), 10, mpp=mpp)
+
+        tiler.extract(slide)
+
+        assert tiler.tile_size == fixed_tile_size
+
 
 class DescribeGridTiler:
     @pytest.mark.parametrize(
-        "fixture_slide, binary_mask, tile_size, level,check_tissue, expectation",
+        "fixture_slide, binary_mask, tile_size, level, check_tissue, expectation",
         [
             (
                 SVS.CMU_1_SMALL_REGION,
@@ -299,6 +355,21 @@ class DescribeGridTiler:
 
         np.testing.assert_array_almost_equal(tiles_location_img, expected_img)
 
+    @pytest.mark.parametrize(
+        "mpp, fixed_tile_size, fixed_overlap",
+        ((None, (512, 512), 32), (0.75, (769, 769), 48)),
+    )
+    def it_sets_tile_size_and_overlap_in_extract_if_mpp(
+        self, mpp, fixed_tile_size, fixed_overlap, tmpdir
+    ):
+        slide = Slide(SVS.CMU_1_SMALL_REGION, tmpdir, use_largeimage=True)
+        tiler = GridTiler((512, 512), pixel_overlap=32, mpp=mpp)
+
+        tiler.extract(slide)
+
+        assert tiler.tile_size == fixed_tile_size
+        assert tiler.pixel_overlap == fixed_overlap
+
 
 class DescribeScoreTiler:
     @pytest.mark.parametrize(
