@@ -9,12 +9,14 @@ import numpy as np
 import openslide
 import PIL
 import pytest
+from large_image.exceptions import TileSourceException
 from PIL import ImageShow
 
-from histolab.exceptions import LevelError, SlidePropertyError
+from histolab.exceptions import LevelError, MayNeedLargeImageError, SlidePropertyError
 from histolab.slide import Slide, SlideSet
 from histolab.types import CP
 
+from ..fixtures import SVS
 from ..unitutil import (
     ANY,
     PILIMG,
@@ -33,18 +35,21 @@ from ..unitutil import (
 
 class Describe_Slide:
     @pytest.mark.parametrize(
-        "slide_path, processed_path",
+        "slide_path, processed_path, use_largeimage",
         [
-            ("/foo/bar/myslide.svs", "/foo/bar/myslide/processed"),
-            (Path("/foo/bar/myslide.svs"), Path("/foo/bar/myslide/processed")),
+            ("/foo/bar/myslide.svs", "/foo/bar/myslide/processed", False),
+            ("/foo/bar/myslide.svs", "/foo/bar/myslide/processed", True),
+            (Path("/foo/bar/myslide.svs"), Path("/foo/bar/myslide/processed"), False),
         ],
     )
-    def it_constructs_from_args(self, request, slide_path, processed_path):
+    def it_constructs_from_args(
+        self, request, slide_path, processed_path, use_largeimage
+    ):
         _init_ = initializer_mock(request, Slide)
 
-        slide = Slide(slide_path, processed_path)
+        slide = Slide(slide_path, processed_path, use_largeimage=use_largeimage)
 
-        _init_.assert_called_once_with(ANY, slide_path, processed_path)
+        _init_.assert_called_once_with(ANY, slide_path, processed_path, use_largeimage)
         assert isinstance(slide, Slide)
 
     def but_it_has_wrong_slide_path_type(self):
@@ -97,6 +102,77 @@ class Describe_Slide:
 
         assert name == expected_value
 
+    @pytest.mark.parametrize(
+        "use_largeimage, properties, metadata, expected_value",
+        (
+            (True, {}, {"mm_x": 1}, 1000.0),
+            (False, {"openslide.mpp-x": 33}, None, 33.0),
+            (False, {"aperio.MPP": 33}, None, 33.0),
+            (
+                False,
+                {"tiff.XResolution": 1000, "tiff.ResolutionUnit": "centimeter"},
+                None,
+                10.0,
+            ),
+        ),
+    )
+    def it_knows_its_base_mpp(
+        self, request, use_largeimage, properties, metadata, expected_value
+    ):
+        slide = Slide("foo", "bar", use_largeimage=use_largeimage)
+        property_mock(request, Slide, "properties", return_value=properties)
+        property_mock(request, Slide, "_metadata", return_value=metadata)
+
+        assert slide.base_mpp == expected_value
+
+    def but_it_raises_large_image_error_with_unknown_mpp_without_largeimage(
+        self, request
+    ):
+        slide = Slide("foo", "bar", use_largeimage=False)
+        property_mock(request, Slide, "properties", return_value={})
+
+        with pytest.raises(MayNeedLargeImageError) as err:
+            slide.base_mpp
+
+        assert str(err.value) == (
+            "Unknown scan magnification! This slide format may be best "
+            "handled using the large_image module. Consider setting "
+            "use_largeimage to True when instantiating this Slide."
+        )
+
+    def and_it_raises_value_error_with_unknown_mpp_with_largeimage(self, request):
+        slide = Slide("foo", "bar", use_largeimage=True)
+        property_mock(request, Slide, "_metadata", return_value={})
+
+        with pytest.raises(ValueError) as err:
+            slide.base_mpp
+
+        assert str(err.value) == (
+            "Unknown scan resolution! This slide is missing metadata "
+            "needed for calculating the scanning resolution. Without "
+            "this information, you can only ask for a tile by level, "
+            "not mpp resolution."
+        )
+
+    def it_has_largeimage_tilesource(self, tmpdir):
+        slide, _ = base_test_slide(
+            tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240, use_largeimage=True
+        )
+
+        assert slide._tile_source.name == "pilfile"
+
+    def it_raises_error_if_tilesource_and_not_use_largeimage(self):
+        slide = Slide("/a/b/foo", "processed")
+
+        with pytest.raises(MayNeedLargeImageError) as err:
+            slide._tile_source
+
+        assert isinstance(err.value, MayNeedLargeImageError)
+        assert str(err.value) == (
+            "This property uses the large_image module. Please set "
+            "use_largeimage to True when instantiating this Slide."
+        )
+
     def it_knows_its_dimensions(self, tmpdir):
         slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
 
@@ -104,6 +180,41 @@ class Describe_Slide:
 
         assert slide_dims == (500, 500)
 
+    def it_extracts_tile_with_mpp_without_image_resizing(self, tmpdir, request):
+        slide = Slide(SVS.CMU_1_SMALL_REGION, tmpdir, use_largeimage=True)
+        property_mock(request, PIL.Image.Image, "size", return_value=(128, 128))
+
+        tile = slide.extract_tile(CP(0, 10, 0, 10), (128, 128), level=None, mpp=0.25)
+
+        assert tile.image.size == (128, 128)
+
+    def but_it_raises_a_runtime_error_when_tile_size_and_mpp_are_not_compatible(
+        self, tmpdir, request
+    ):
+        slide = Slide(SVS.CMU_1_SMALL_REGION, tmpdir, use_largeimage=True)
+        property_mock(request, PIL.Image.Image, "size", return_value=(100, 100))
+
+        with pytest.raises(RuntimeError) as err:
+            slide.extract_tile(CP(0, 10, 0, 10), (128, 128), level=None, mpp=0.25)
+
+        assert (
+            str(err.value)
+            == "The tile you requested at a resolution of 0.25 MPP has a size of "
+            "(100, 100), yet you specified a final `tile_size` of (128, 128), "
+            "which is a very different value. When you set `mpp`, "
+            "the `tile_size` parameter is used to resize fetched tiles if they are "
+            "off by just 5 pixels due to rounding differences etc. Please check if "
+            "you requested the right `mpp` and/or `tile_size`."
+        )
+
+    def it_raises_error_if_bad_args_for_extract_tile(self):
+        slide = Slide("foo", "bar", use_largeimage=True)
+
+        with pytest.raises(ValueError) as err:
+            slide.extract_tile(CP(0, 10, 0, 10), (10, 10), level=None, mpp=None)
+
+        assert str(err.value) == "Either level or mpp must be provided!"
+
     def it_knows_its_resampled_dimensions(self, dimensions_):
         """This test prove that given the dimensions (mock object here), it does
         the correct maths operations:
@@ -148,6 +259,14 @@ class Describe_Slide:
 
         assert thumb_size == (500, 500)
 
+    def it_raises_error_if_thumbnail_size_and_use_largeimage(self):
+        slide = Slide("/a/b/foo", "processed", use_largeimage=True)
+
+        with pytest.raises(TileSourceException) as err:
+            slide._thumbnail_size
+
+        assert str(err.value) == "No available tilesource for /a/b/foo"
+
     def it_creates_a_correct_slide_object(self, tmpdir):
         slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_50X50_155_0_0)
 
@@ -173,12 +292,23 @@ class Describe_Slide:
             slide._wsi
 
         assert isinstance(err.value, PIL.UnidentifiedImageError)
-        assert (
-            str(err.value) == "Your wsi has something broken inside, a doctor is needed"
+        assert str(err.value) == (
+            "This slide may be corrupted or have a non-standard format not "
+            "handled by the openslide and PIL libraries. Consider setting "
+            "use_largeimage to True when instantiating this Slide."
         )
 
-    def it_can_resample_itself(self, tmpdir, resampled_dims_):
-        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
+    @pytest.mark.parametrize(
+        "use_largeimage",
+        [
+            (False,),
+            (True,),
+        ],
+    )
+    def it_can_resample_itself(self, tmpdir, resampled_dims_, use_largeimage):
+        slide, _ = base_test_slide(
+            tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240, use_largeimage=use_largeimage
+        )
         resampled_dims_.return_value = (100, 200, 300, 400)
 
         _resample = slide._resample(32)
@@ -222,12 +352,23 @@ class Describe_Slide:
 
         assert type(scaled_image) == PIL.Image.Image
 
-    def it_knows_its_thumbnail(self, tmpdir, resampled_dims_):
+    @pytest.mark.parametrize(
+        "use_largeimage",
+        [
+            (False,),
+            (True,),
+        ],
+    )
+    def it_knows_its_thumbnail(self, tmpdir, resampled_dims_, use_largeimage):
         tmp_path_ = tmpdir.mkdir("myslide")
         image = PILIMG.RGBA_COLOR_500X500_155_249_240
         image.save(os.path.join(tmp_path_, "mywsi.png"), "PNG")
         slide_path = os.path.join(tmp_path_, "mywsi.png")
-        slide = Slide(slide_path, os.path.join(tmp_path_, "processed"))
+        slide = Slide(
+            slide_path,
+            os.path.join(tmp_path_, "processed"),
+            use_largeimage=use_largeimage,
+        )
         resampled_dims_.return_value = (100, 200, 300, 400)
 
         thumb = slide.thumbnail
@@ -453,15 +594,28 @@ class Describe_Slideset:
         _slides_path = "/foo/bar/"
         _processed_path = "/foo/bar/wsislides/processed"
         _valid_extensions = [".svs", ".tiff"]
+        _keep_slides = ["mywsi.svs"]
+        _slide_kwargs = {"use_largeimage": True}
 
-        slideset = SlideSet(_slides_path, _processed_path, _valid_extensions)
+        slideset = SlideSet(
+            _slides_path,
+            _processed_path,
+            _valid_extensions,
+            _keep_slides,
+            _slide_kwargs,
+        )
 
         _init_.assert_called_once_with(
-            ANY, _slides_path, _processed_path, _valid_extensions
+            ANY,
+            _slides_path,
+            _processed_path,
+            _valid_extensions,
+            _keep_slides,
+            _slide_kwargs,
         )
         assert isinstance(slideset, SlideSet)
 
-    def it_can_constructs_slides(self, request, tmpdir, Slide_):
+    def it_can_construct_slides(self, request, tmpdir, Slide_):
         tmp_path_ = tmpdir.mkdir("myslide")
         slides_ = method_mock(request, SlideSet, "__iter__")
         slides_.return_value = [Slide_ for _ in range(10)]
@@ -475,9 +629,15 @@ class Describe_Slideset:
     def it_knows_its_slides(self, tmpdir):
         tmp_path_ = tmpdir.mkdir("myslide")
         image = PILIMG.RGBA_COLOR_500X500_155_249_240
-        image.save(os.path.join(tmp_path_, "mywsi.svs"), "TIFF")
+        image.save(os.path.join(tmp_path_, "mywsi1.svs"), "TIFF")
+        image.save(os.path.join(tmp_path_, "mywsi2.svs"), "TIFF")
         slideset = SlideSet(tmp_path_, "proc", [".svs"])
 
+        assert len(slideset) == 2
+
+        # it can keep a subset of slides
+        slideset = SlideSet(tmp_path_, "proc", [".svs"], keep_slides=["mywsi1.svs"])
+
         assert len(slideset) == 1
 
         slideset = SlideSet(None, "proc", [".svs"])
@@ -491,6 +651,22 @@ class Describe_Slideset:
         assert isinstance(err.value, FileNotFoundError)
         assert err.value.errno == errno.ENOENT
 
+    @pytest.mark.parametrize("slide_kwargs", (({"use_largeimage": True}), ({})))
+    def it_creates_its_slides_with_the_correct_parameters(
+        self, tmpdir, request, slide_kwargs
+    ):
+        slide_init_ = initializer_mock(request, Slide)
+        tmp_path_ = tmpdir.mkdir("myslide")
+        image = PILIMG.RGBA_COLOR_500X500_155_249_240
+        image.save(os.path.join(tmp_path_, "mywsi1.svs"), "TIFF")
+        slideset = SlideSet(tmp_path_, "proc", [".svs"], slide_kwargs=slide_kwargs)
+
+        slideset.__iter__()
+
+        slide_init_.assert_called_once_with(
+            ANY, os.path.join(tmp_path_, "mywsi1.svs"), "proc", **slide_kwargs
+        )
+
     def it_can_access_directly_to_the_slides(self, request, Slide_):
         slideset = instance_mock(request, SlideSet)
         slideset.__iter__.side_effect = iter([Slide_])
